-- app size
local myappwidth, myappheight = application:getContentWidth(), application:getContentHeight()

-- controls
local isup, isdown, isleft, isright, isshoot = false, false, false, false, false

-- category bits
local PLAYER1_BIT = math.pow(2, 0)
local ENEMY_BIT = math.pow(2, 1)
local MISSILE_BIT = math.pow(2, 2)

-- r3d
r3d = require "reactphysics3d"
local gravityy = 0
local world = r3d.World.new(0, gravityy, 0) -- 0, -9.8, 0
world.missiles = {}

-- camera
local camera = D3.View.new(myappwidth, myappheight, 60, 0.01, 1024)

-- a sky
local sky = Pixel.new(0xffffff, 1, myappwidth * 2, myappheight * 2)
sky:setAnchorPoint(0.5, 0.5)
sky:setPosition(myappwidth / 2, myappheight / 2)
sky:setColor(0x0011ff, 1 , 0x0099ff, 1, 90)
stage:addChild(sky)

-- the player
local playerx = PLAYER1.new(world, "3d_models/mkship", "mkship.obj", PLAYER1_BIT, ENEMY_BIT)
player = playerx:getObj()
playerbody = playerx:getObjBody()

-- an enemy
local enemy01x = OBJS.new(world, "3d_models/enemy", "ship03.obj", ENEMY_BIT, PLAYER1_BIT)
enemy01 = enemy01x:getObj()
enemy01body = enemy01x:getObjBody()
enemy01x:setObjBodyMatrix(0, 4, 12,	0, 180, 0) -- pos, rot

-- scene
camera:getScene():addChild(player)
camera:getScene():addChild(enemy01)
camera:lookAt(-12, 0, 0, player:getPosition())
stage:addChild(camera)

-- let's go!
local speed = 20*8
local m1 = Matrix.new()
stage:addEventListener(Event.ENTER_FRAME, function(e)
	-- remove missiles
	for k, v in ipairs(world.missiles) do
		m1 = v:getTransform()
		k:setMatrix(v:getTransform())
		if m1:getZ() > 9 then
			world:destroyBody(v)
			camera:getScene():removeChild(k)
			world.missiles[k] = nil
		end
	end

	-- action
	world:step(e.deltaTime)

	-- player 1
	if isup then playerbody:applyForce(0, speed, 0) end
	if isdown then playerbody:applyForce(0, -speed, 0) end
	if isleft then playerbody:applyForce(0, 0, speed * 6) end
	if isright then playerbody:applyForce(0, 0, -speed * 6) end
	if isshoot then shoot() isshoot = false end
	player:setMatrix(playerbody:getTransform())

	-- enemy
	enemy01body:applyForce(0, 0, -speed*2.5)
	enemy01:setMatrix(enemy01body:getTransform())
	m1 = enemy01body:getTransform()
	if m1:getZ() < -10 then enemy01x:setObjBodyMatrix(0,4,16,	0,180,0) end

	-- missiles
	for k, v in ipairs(world.missiles) do
		camera:getScene():addChild(k)
		v:applyForce(0,0,speed)
		k:setMatrix(v:getTransform())
	end
end)

function shoot()
	local missile, missileobj = MISSILES(world, "3d_models/missiles", "missile03.obj", MISSILE_BIT, ENEMY_BIT)
	world.missiles[missile] = missileobj
	print(#world.missiles)
end

-- keyboard
stage:addEventListener(Event.KEY_DOWN, function(e)
	if e.keyCode == KeyCode.UP then isup = true end
	if e.keyCode == KeyCode.DOWN then isdown = true end
	if e.keyCode == KeyCode.LEFT then isleft = true end
	if e.keyCode == KeyCode.RIGHT then isright = true end
	if e.keyCode == KeyCode.SPACE then isshoot = true end
end)

stage:addEventListener(Event.KEY_UP, function(e)
	if e.keyCode == KeyCode.UP then isup = false end
	if e.keyCode == KeyCode.DOWN then isdown = false end
	if e.keyCode == KeyCode.LEFT then isleft = false end
	if e.keyCode == KeyCode.RIGHT then isright = false end
	if e.keyCode == KeyCode.SPACE then isshoot = false end
end)
